@@ -70,7 +70,7 @@ Q_SIGNALS:
     void error(QRadioData::Error err);
 
 protected:
-    QRadioDataControl(QObject *parent = 0);
+    explicit QRadioDataControl(QObject *parent = Q_NULLPTR);
 };
 
 #define QRadioDataControl_iid "org.qt-project.qt.radiodatacontrol/5.0"
