@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
+** Copyright (C) 2016 The Qt Company Ltd.
 ** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
@@ -45,6 +45,7 @@
 #include <QtCore/qvector.h>
 #include <QtCore/qdebug.h>
 #include <QtCore/qlist.h>
+#include <private/qmultimediautils_p.h>
 
 #include <algorithm>
 
@@ -65,201 +66,10 @@ bool qt_framerates_sane(const QCameraViewfinderSettings &settings)
     return !maxFPS || maxFPS >= minFPS;
 }
 
-void qt_set_framerate_limits(AVCaptureConnection *videoConnection,
-                             const QCameraViewfinderSettings &settings)
-{
-    Q_ASSERT(videoConnection);
-
-    if (!qt_framerates_sane(settings)) {
-        qDebugCamera() << Q_FUNC_INFO << "invalid framerate (min, max):"
-                       << settings.minimumFrameRate() << settings.maximumFrameRate();
-        return;
-    }
-
-    const qreal maxFPS = settings.maximumFrameRate();
-    CMTime minDuration = kCMTimeInvalid;
-    if (maxFPS > 0.) {
-        if (!videoConnection.supportsVideoMinFrameDuration)
-            qDebugCamera() << Q_FUNC_INFO << "maximum framerate is not supported";
-        else
-            minDuration = CMTimeMake(1, maxFPS);
-    }
-    if (videoConnection.supportsVideoMinFrameDuration)
-        videoConnection.videoMinFrameDuration = minDuration;
-
-    const qreal minFPS = settings.minimumFrameRate();
-#if QT_MAC_PLATFORM_SDK_EQUAL_OR_ABOVE(__MAC_10_9, __IPHONE_5_0)
-#if QT_OSX_DEPLOYMENT_TARGET_BELOW(__MAC_10_9)
-    if (QSysInfo::MacintoshVersion < QSysInfo::MV_10_9) {
-        if (minFPS > 0.)
-            qDebugCamera() << Q_FUNC_INFO << "minimum framerate is not supported";
-    } else
-#endif
-    {
-        CMTime maxDuration = kCMTimeInvalid;
-        if (minFPS > 0.) {
-            if (!videoConnection.supportsVideoMaxFrameDuration)
-                qDebugCamera() << Q_FUNC_INFO << "minimum framerate is not supported";
-            else
-                maxDuration = CMTimeMake(1, minFPS);
-        }
-        if (videoConnection.supportsVideoMaxFrameDuration)
-            videoConnection.videoMaxFrameDuration = maxDuration;
-    }
-#else
-    if (minFPS > 0.)
-        qDebugCamera() << Q_FUNC_INFO << "minimum framerate is not supported";
-#endif
-}
-
-#if QT_MAC_PLATFORM_SDK_EQUAL_OR_ABOVE(__MAC_10_7, __IPHONE_7_0)
-
-CMTime qt_adjusted_frame_duration(AVFrameRateRange *range, qreal fps)
-{
-    Q_ASSERT(range);
-    Q_ASSERT(fps > 0.);
-
-    if (range.maxFrameRate - range.minFrameRate < 0.1) {
-        // Can happen on OS X.
-        return range.minFrameDuration;
-    }
-
-    if (fps <= range.minFrameRate)
-        return range.maxFrameDuration;
-    if (fps >= range.maxFrameRate)
-        return range.minFrameDuration;
-
-    const AVFRational timeAsRational(qt_float_to_rational(1. / fps, 1000));
-    return CMTimeMake(timeAsRational.first, timeAsRational.second);
-}
-
-void qt_set_framerate_limits(AVCaptureDevice *captureDevice,
-                             const QCameraViewfinderSettings &settings)
-{
-    Q_ASSERT(captureDevice);
-    if (!captureDevice.activeFormat) {
-        qDebugCamera() << Q_FUNC_INFO << "no active capture device format";
-        return;
-    }
-
-    const qreal minFPS = settings.minimumFrameRate();
-    const qreal maxFPS = settings.maximumFrameRate();
-    if (!qt_framerates_sane(settings)) {
-        qDebugCamera() << Q_FUNC_INFO << "invalid framerates (min, max):"
-                       << minFPS << maxFPS;
-        return;
-    }
-
-    CMTime minFrameDuration = kCMTimeInvalid;
-    CMTime maxFrameDuration = kCMTimeInvalid;
-    if (maxFPS || minFPS) {
-        AVFrameRateRange *range = qt_find_supported_framerate_range(captureDevice.activeFormat,
-                                                                    maxFPS ? maxFPS : minFPS);
-        if (!range) {
-            qDebugCamera() << Q_FUNC_INFO << "no framerate range found, (min, max):"
-                           << minFPS << maxFPS;
-            return;
-        }
-
-        if (maxFPS)
-            minFrameDuration = qt_adjusted_frame_duration(range, maxFPS);
-        if (minFPS)
-            maxFrameDuration = qt_adjusted_frame_duration(range, minFPS);
-    }
-
-    const AVFConfigurationLock lock(captureDevice);
-    if (!lock) {
-        qDebugCamera() << Q_FUNC_INFO << "failed to lock for configuration";
-        return;
-    }
-
-    // While Apple's docs say kCMTimeInvalid will end in default
-    // settings for this format, kCMTimeInvalid on OS X ends with a runtime
-    // exception:
-    // "The activeVideoMinFrameDuration passed is not supported by the device."
-#ifdef Q_OS_IOS
-    [captureDevice setActiveVideoMinFrameDuration:minFrameDuration];
-    [captureDevice setActiveVideoMaxFrameDuration:maxFrameDuration];
-#else // Q_OS_OSX
-
-    if (CMTimeCompare(minFrameDuration, kCMTimeInvalid))
-        [captureDevice setActiveVideoMinFrameDuration:minFrameDuration];
-
-#if QT_OSX_PLATFORM_SDK_EQUAL_OR_ABOVE(__MAC_10_9)
-#if QT_OSX_DEPLOYMENT_TARGET_BELOW(__MAC_10_9)
-    if (QSysInfo::MacintoshVersion >= QSysInfo::MV_10_9)
-#endif
-    {
-        if (CMTimeCompare(maxFrameDuration, kCMTimeInvalid))
-            [captureDevice setActiveVideoMaxFrameDuration:maxFrameDuration];
-    }
-#endif // QT_OSX_PLATFORM_SDK_EQUAL_OR_ABOVE(__MAC_10_9)
-#endif // Q_OS_OSX
-}
-
-#endif // Platform SDK >= 10.9, >= 7.0.
-
-// 'Dispatchers':
-
-AVFPSRange qt_current_framerates(AVCaptureDevice *captureDevice, AVCaptureConnection *videoConnection)
-{
-    Q_ASSERT(captureDevice);
-    Q_ASSERT(videoConnection);
-
-    AVFPSRange fps;
-#if QT_MAC_PLATFORM_SDK_EQUAL_OR_ABOVE(__MAC_10_7, __IPHONE_7_0)
-    if (QSysInfo::MacintoshVersion >= qt_OS_limit(QSysInfo::MV_10_7, QSysInfo::MV_IOS_7_0)) {
-        const CMTime minDuration = captureDevice.activeVideoMinFrameDuration;
-        if (CMTimeCompare(minDuration, kCMTimeInvalid)) {
-            if (const Float64 minSeconds = CMTimeGetSeconds(minDuration))
-                fps.second = 1. / minSeconds; // Max FPS = 1 / MinDuration.
-        }
-
-#if QT_OSX_PLATFORM_SDK_EQUAL_OR_ABOVE(__MAC_10_9)
-#if QT_OSX_DEPLOYMENT_TARGET_BELOW(__MAC_10_9)
-        if (QSysInfo::MacintoshVersion >= QSysInfo::MV_10_9)
-#endif
-        {
-            const CMTime maxDuration = captureDevice.activeVideoMaxFrameDuration;
-            if (CMTimeCompare(maxDuration, kCMTimeInvalid)) {
-                if (const Float64 maxSeconds = CMTimeGetSeconds(maxDuration))
-                    fps.first = 1. / maxSeconds; // Min FPS = 1 / MaxDuration.
-            }
-        }
-#endif // QT_OSX_PLATFORM_SDK_EQUAL_OR_ABOVE(__MAC_10_9)
-
-    } else {
-#else // OSX < 10.7 or iOS < 7.0
-    {
-#endif // QT_MAC_PLATFORM_SDK_EQUAL_OR_ABOVE(__MAC_10_7, __IPHONE_7_0)
-        fps = qt_connection_framerates(videoConnection);
-    }
-
-    return fps;
-}
-
-void qt_set_framerate_limits(AVCaptureDevice *captureDevice, AVCaptureConnection *videoConnection,
-                             const QCameraViewfinderSettings &settings)
-{
-    Q_ASSERT(captureDevice);
-    Q_ASSERT(videoConnection);
-#if QT_MAC_PLATFORM_SDK_EQUAL_OR_ABOVE(__MAC_10_7, __IPHONE_7_0)
-    if (QSysInfo::MacintoshVersion >= qt_OS_limit(QSysInfo::MV_10_9, QSysInfo::MV_IOS_7_0))
-        qt_set_framerate_limits(captureDevice, settings);
-    else
-        qt_set_framerate_limits(videoConnection, settings);
-#else
-    qt_set_framerate_limits(videoConnection, settings);
-#endif
-}
-
 } // Unnamed namespace.
 
 AVFCameraViewfinderSettingsControl2::AVFCameraViewfinderSettingsControl2(AVFCameraService *service)
-    : m_service(service),
-      m_captureDevice(0),
-      m_videoOutput(0),
-      m_videoConnection(0)
+    : m_service(service)
 {
     Q_ASSERT(service);
 }
@@ -268,8 +78,9 @@ QList<QCameraViewfinderSettings> AVFCameraViewfinderSettingsControl2::supportedV
 {
     QList<QCameraViewfinderSettings> supportedSettings;
 
-    if (!updateAVFoundationObjects()) {
-        qDebugCamera() << Q_FUNC_INFO << "no capture device or video output found";
+    AVCaptureDevice *captureDevice = m_service->session()->videoCaptureDevice();
+    if (!captureDevice) {
+        qDebugCamera() << Q_FUNC_INFO << "no capture device found";
         return supportedSettings;
     }
 
@@ -279,15 +90,16 @@ QList<QCameraViewfinderSettings> AVFCameraViewfinderSettingsControl2::supportedV
 
     if (!pixelFormats.size())
         pixelFormats << QVideoFrame::Format_Invalid; // The default value.
+
 #if QT_MAC_PLATFORM_SDK_EQUAL_OR_ABOVE(__MAC_10_7, __IPHONE_7_0)
     if (QSysInfo::MacintoshVersion >= qt_OS_limit(QSysInfo::MV_10_7, QSysInfo::MV_IOS_7_0)) {
-        if (!m_captureDevice.formats || !m_captureDevice.formats.count) {
+        if (!captureDevice.formats || !captureDevice.formats.count) {
             qDebugCamera() << Q_FUNC_INFO << "no capture device formats found";
             return supportedSettings;
         }
 
-        const QVector<AVCaptureDeviceFormat *> formats(qt_unique_device_formats(m_captureDevice,
-                                                       m_session->defaultCodec()));
+        const QVector<AVCaptureDeviceFormat *> formats(qt_unique_device_formats(captureDevice,
+                                                       m_service->session()->defaultCodec()));
         for (int i = 0; i < formats.size(); ++i) {
             AVCaptureDeviceFormat *format = formats[i];
 
@@ -318,15 +130,18 @@ QList<QCameraViewfinderSettings> AVFCameraViewfinderSettingsControl2::supportedV
 #else
     {
 #endif
-        // TODO: resolution and PAR.
-        framerates << qt_connection_framerates(m_videoConnection);
-        for (int i = 0; i < pixelFormats.size(); ++i) {
-            for (int j = 0; j < framerates.size(); ++j) {
-                QCameraViewfinderSettings newSet;
-                newSet.setPixelFormat(pixelFormats[i]);
-                newSet.setMinimumFrameRate(framerates[j].first);
-                newSet.setMaximumFrameRate(framerates[j].second);
-                supportedSettings << newSet;
+        AVCaptureConnection *connection = videoConnection();
+        if (connection) {
+            // TODO: resolution and PAR.
+            framerates << qt_connection_framerates(connection);
+            for (int i = 0; i < pixelFormats.size(); ++i) {
+                for (int j = 0; j < framerates.size(); ++j) {
+                    QCameraViewfinderSettings newSet;
+                    newSet.setPixelFormat(pixelFormats[i]);
+                    newSet.setMinimumFrameRate(framerates[j].first);
+                    newSet.setMaximumFrameRate(framerates[j].second);
+                    supportedSettings << newSet;
+                }
             }
         }
     }
@@ -336,22 +151,28 @@ QList<QCameraViewfinderSettings> AVFCameraViewfinderSettingsControl2::supportedV
 
 QCameraViewfinderSettings AVFCameraViewfinderSettingsControl2::viewfinderSettings() const
 {
-    QCameraViewfinderSettings settings;
+    QCameraViewfinderSettings settings = m_settings;
 
-    if (!updateAVFoundationObjects()) {
-        qDebugCamera() << Q_FUNC_INFO << "no capture device or video output found";
+    AVCaptureDevice *captureDevice = m_service->session()->videoCaptureDevice();
+    if (!captureDevice) {
+        qDebugCamera() << Q_FUNC_INFO << "no capture device found";
+        return settings;
+    }
+
+    if (m_service->session()->state() != QCamera::LoadedState &&
+        m_service->session()->state() != QCamera::ActiveState) {
         return settings;
     }
 
 #if QT_MAC_PLATFORM_SDK_EQUAL_OR_ABOVE(__MAC_10_7, __IPHONE_7_0)
     if (QSysInfo::MacintoshVersion >= qt_OS_limit(QSysInfo::MV_10_7, QSysInfo::MV_IOS_7_0)) {
-        if (!m_captureDevice.activeFormat) {
+        if (!captureDevice.activeFormat) {
             qDebugCamera() << Q_FUNC_INFO << "no active capture device format";
             return settings;
         }
 
-        const QSize res(qt_device_format_resolution(m_captureDevice.activeFormat));
-        const QSize par(qt_device_format_pixel_aspect_ratio(m_captureDevice.activeFormat));
+        const QSize res(qt_device_format_resolution(captureDevice.activeFormat));
+        const QSize par(qt_device_format_pixel_aspect_ratio(captureDevice.activeFormat));
         if (res.isNull() || !res.isValid() || par.isNull() || !par.isValid()) {
             qDebugCamera() << Q_FUNC_INFO << "failed to obtain resolution/pixel aspect ratio";
             return settings;
@@ -362,12 +183,14 @@ QCameraViewfinderSettings AVFCameraViewfinderSettingsControl2::viewfinderSetting
     }
 #endif
     // TODO: resolution and PAR before 7.0.
-    const AVFPSRange fps = qt_current_framerates(m_captureDevice, m_videoConnection);
+    const AVFPSRange fps = qt_current_framerates(captureDevice, videoConnection());
     settings.setMinimumFrameRate(fps.first);
     settings.setMaximumFrameRate(fps.second);
 
-    if (NSObject *obj = [m_videoOutput.videoSettings objectForKey:(id)kCVPixelBufferPixelFormatTypeKey]) {
-        if ([obj isKindOfClass:[NSNumber class]]) {
+    AVCaptureVideoDataOutput *videoOutput = m_service->videoOutput() ? m_service->videoOutput()->videoDataOutput() : 0;
+    if (videoOutput) {
+        NSObject *obj = [videoOutput.videoSettings objectForKey:(id)kCVPixelBufferPixelFormatTypeKey];
+        if (obj && [obj isKindOfClass:[NSNumber class]]) {
             NSNumber *nsNum = static_cast<NSNumber *>(obj);
             settings.setPixelFormat(QtPixelFormatFromCVFormat([nsNum unsignedIntValue]));
         }
@@ -378,16 +201,11 @@ QCameraViewfinderSettings AVFCameraViewfinderSettingsControl2::viewfinderSetting
 
 void AVFCameraViewfinderSettingsControl2::setViewfinderSettings(const QCameraViewfinderSettings &settings)
 {
-    if (settings.isNull()) {
-        qDebugCamera() << Q_FUNC_INFO << "empty viewfinder settings";
-        return;
-    }
-
     if (m_settings == settings)
         return;
 
     m_settings = settings;
-    applySettings();
+    applySettings(m_settings);
 }
 
 QVideoFrame::PixelFormat AVFCameraViewfinderSettingsControl2::QtPixelFormatFromCVFormat(unsigned avPixelFormat)
@@ -447,17 +265,19 @@ bool AVFCameraViewfinderSettingsControl2::CVPixelFormatFromQtFormat(QVideoFrame:
 
 AVCaptureDeviceFormat *AVFCameraViewfinderSettingsControl2::findBestFormatMatch(const QCameraViewfinderSettings &settings) const
 {
+    AVCaptureDevice *captureDevice = m_service->session()->videoCaptureDevice();
+    if (!captureDevice || settings.isNull())
+        return nil;
+
 #if QT_MAC_PLATFORM_SDK_EQUAL_OR_ABOVE(__MAC_10_7, __IPHONE_7_0)
     if (QSysInfo::MacintoshVersion >= qt_OS_limit(QSysInfo::MV_10_7, QSysInfo::MV_IOS_7_0)) {
-        Q_ASSERT(m_captureDevice);
-        Q_ASSERT(m_session);
 
         const QSize &resolution = settings.resolution();
         if (!resolution.isNull() && resolution.isValid()) {
             // Either the exact match (including high resolution for images on iOS)
             // or a format with a resolution close to the requested one.
-            return qt_find_best_resolution_match(m_captureDevice, resolution,
-                                                 m_session->defaultCodec());
+            return qt_find_best_resolution_match(captureDevice, resolution,
+                                                 m_service->session()->defaultCodec());
         }
 
         // No resolution requested, what about framerates?
@@ -470,22 +290,29 @@ AVCaptureDeviceFormat *AVFCameraViewfinderSettingsControl2::findBestFormatMatch(
         const qreal minFPS(settings.minimumFrameRate());
         const qreal maxFPS(settings.maximumFrameRate());
         if (minFPS || maxFPS)
-            return qt_find_best_framerate_match(m_captureDevice, maxFPS ? maxFPS : minFPS,
-                                                m_session->defaultCodec());
+            return qt_find_best_framerate_match(captureDevice,
+                                                m_service->session()->defaultCodec(),
+                                                maxFPS ? maxFPS : minFPS);
         // Ignore PAR for the moment (PAR without resolution can
         // pick a format with really bad resolution).
         // No need to test pixel format, just return settings.
     }
 #endif
+
     return nil;
 }
 
 QVector<QVideoFrame::PixelFormat> AVFCameraViewfinderSettingsControl2::viewfinderPixelFormats() const
 {
-    Q_ASSERT(m_videoOutput);
-
     QVector<QVideoFrame::PixelFormat> qtFormats;
-    NSArray *pixelFormats = [m_videoOutput availableVideoCVPixelFormatTypes];
+
+    AVCaptureVideoDataOutput *videoOutput = m_service->videoOutput() ? m_service->videoOutput()->videoDataOutput() : 0;
+    if (!videoOutput) {
+        qDebugCamera() << Q_FUNC_INFO << "no video output found";
+        return qtFormats;
+    }
+
+    NSArray *pixelFormats = [videoOutput availableVideoCVPixelFormatTypes];
 
     for (NSObject *obj in pixelFormats) {
         if (![obj isKindOfClass:[NSNumber class]])
@@ -506,17 +333,19 @@ QVector<QVideoFrame::PixelFormat> AVFCameraViewfinderSettingsControl2::viewfinde
 bool AVFCameraViewfinderSettingsControl2::convertPixelFormatIfSupported(QVideoFrame::PixelFormat qtFormat,
                                                                         unsigned &avfFormat)const
 {
-    Q_ASSERT(m_videoOutput);
+    AVCaptureVideoDataOutput *videoOutput = m_service->videoOutput() ? m_service->videoOutput()->videoDataOutput() : 0;
+    if (!videoOutput)
+        return false;
 
     unsigned conv = 0;
     if (!CVPixelFormatFromQtFormat(qtFormat, conv))
         return false;
 
-    NSArray *formats = [m_videoOutput availableVideoCVPixelFormatTypes];
+    NSArray *formats = [videoOutput availableVideoCVPixelFormatTypes];
     if (!formats || !formats.count)
         return false;
 
-    if (m_service->videoOutput() && m_service->videoOutput()->surface()) {
+    if (m_service->videoOutput()->surface()) {
         const QAbstractVideoSurface *surface = m_service->videoOutput()->surface();
         if (!surface->supportedPixelFormats().contains(qtFormat))
             return false;
@@ -537,68 +366,71 @@ bool AVFCameraViewfinderSettingsControl2::convertPixelFormatIfSupported(QVideoFr
     return found;
 }
 
-void AVFCameraViewfinderSettingsControl2::applySettings()
+bool AVFCameraViewfinderSettingsControl2::applySettings(const QCameraViewfinderSettings &settings)
 {
-    if (m_settings.isNull())
-        return;
-
-    if (!updateAVFoundationObjects())
-        return;
-
-    if (m_session->state() != QCamera::LoadedState &&
-        m_session->state() != QCamera::ActiveState) {
-        return;
+    if (m_service->session()->state() != QCamera::LoadedState &&
+        m_service->session()->state() != QCamera::ActiveState) {
+        return false;
     }
 
-    NSMutableDictionary *videoSettings = [NSMutableDictionary dictionaryWithCapacity:1];
-#if QT_MAC_PLATFORM_SDK_EQUAL_OR_ABOVE(__MAC_10_7, __IPHONE_7_0)
-    AVCaptureDeviceFormat *match = findBestFormatMatch(m_settings);
-    if (match) {
-        if (match != m_captureDevice.activeFormat) {
-            const AVFConfigurationLock lock(m_captureDevice);
-            if (!lock) {
-                qDebugCamera() << Q_FUNC_INFO << "failed to lock for configuration";
-                return;
-            }
+    AVCaptureDevice *captureDevice = m_service->session()->videoCaptureDevice();
+    if (!captureDevice)
+        return false;
 
-            m_captureDevice.activeFormat = match;
-        }
+    bool activeFormatChanged = false;
+
+#if QT_MAC_PLATFORM_SDK_EQUAL_OR_ABOVE(__MAC_10_7, __IPHONE_7_0)
+    AVCaptureDeviceFormat *match = findBestFormatMatch(settings);
+    if (match) {
+        activeFormatChanged = qt_set_active_format(captureDevice, match, false);
     } else {
         qDebugCamera() << Q_FUNC_INFO << "matching device format not found";
         // We still can update the pixel format at least.
     }
 #endif
 
-    unsigned avfPixelFormat = 0;
-    if (!convertPixelFormatIfSupported(m_settings.pixelFormat(), avfPixelFormat)) {
-        // If the the pixel format is not specified or invalid, pick the preferred video surface
-        // format, or if no surface is set, the preferred capture device format
-        const QVector<QVideoFrame::PixelFormat> deviceFormats = viewfinderPixelFormats();
-        QList<QVideoFrame::PixelFormat> surfaceFormats;
-        if (m_service->videoOutput() && m_service->videoOutput()->surface())
-            surfaceFormats = m_service->videoOutput()->surface()->supportedPixelFormats();
+    AVCaptureVideoDataOutput *videoOutput = m_service->videoOutput() ? m_service->videoOutput()->videoDataOutput() : 0;
+    if (videoOutput) {
+        unsigned avfPixelFormat = 0;
+        if (!convertPixelFormatIfSupported(settings.pixelFormat(), avfPixelFormat)) {
+            // If the the pixel format is not specified or invalid, pick the preferred video surface
+            // format, or if no surface is set, the preferred capture device format
 
-        QVideoFrame::PixelFormat format = deviceFormats.first();
+            const QVector<QVideoFrame::PixelFormat> deviceFormats = viewfinderPixelFormats();
+            QVideoFrame::PixelFormat pickedFormat = deviceFormats.first();
 
-        for (int i = 0; i < surfaceFormats.count(); ++i) {
-            const QVideoFrame::PixelFormat surfaceFormat = surfaceFormats.at(i);
-            if (deviceFormats.contains(surfaceFormat)) {
-                format = surfaceFormat;
-                break;
+            QAbstractVideoSurface *surface = m_service->videoOutput()->surface();
+            if (surface) {
+                if (m_service->videoOutput()->supportsTextures()) {
+                    pickedFormat = QVideoFrame::Format_ARGB32;
+                } else {
+                    QList<QVideoFrame::PixelFormat> surfaceFormats = surface->supportedPixelFormats();
+
+                    for (int i = 0; i < surfaceFormats.count(); ++i) {
+                        const QVideoFrame::PixelFormat surfaceFormat = surfaceFormats.at(i);
+                        if (deviceFormats.contains(surfaceFormat)) {
+                            pickedFormat = surfaceFormat;
+                            break;
+                        }
+                    }
+                }
             }
+
+            CVPixelFormatFromQtFormat(pickedFormat, avfPixelFormat);
         }
 
-        CVPixelFormatFromQtFormat(format, avfPixelFormat);
+        if (avfPixelFormat != 0) {
+            NSMutableDictionary *videoSettings = [NSMutableDictionary dictionaryWithCapacity:1];
+            [videoSettings setObject:[NSNumber numberWithUnsignedInt:avfPixelFormat]
+                                      forKey:(id)kCVPixelBufferPixelFormatTypeKey];
+
+            videoOutput.videoSettings = videoSettings;
+        }
     }
 
-    if (avfPixelFormat != 0) {
-        [videoSettings setObject:[NSNumber numberWithUnsignedInt:avfPixelFormat]
-                          forKey:(id)kCVPixelBufferPixelFormatTypeKey];
+    qt_set_framerate_limits(captureDevice, videoConnection(), settings.minimumFrameRate(), settings.maximumFrameRate());
 
-        m_videoOutput.videoSettings = videoSettings;
-    }
-
-    qt_set_framerate_limits(m_captureDevice, m_videoConnection, m_settings);
+    return activeFormatChanged;
 }
 
 QCameraViewfinderSettings AVFCameraViewfinderSettingsControl2::requestedSettings() const
@@ -606,33 +438,12 @@ QCameraViewfinderSettings AVFCameraViewfinderSettingsControl2::requestedSettings
     return m_settings;
 }
 
-bool AVFCameraViewfinderSettingsControl2::updateAVFoundationObjects() const
+AVCaptureConnection *AVFCameraViewfinderSettingsControl2::videoConnection() const
 {
-    m_session = 0;
-    m_captureDevice = 0;
-    m_videoOutput = 0;
-    m_videoConnection = 0;
-
-    if (!m_service->session())
-        return false;
-
-    if (!m_service->session()->videoCaptureDevice())
-        return false;
-
     if (!m_service->videoOutput() || !m_service->videoOutput()->videoDataOutput())
-        return false;
+        return nil;
 
-    AVCaptureVideoDataOutput *output = m_service->videoOutput()->videoDataOutput();
-    AVCaptureConnection *connection = [output connectionWithMediaType:AVMediaTypeVideo];
-    if (!connection)
-        return false;
-
-    m_session = m_service->session();
-    m_captureDevice = m_session->videoCaptureDevice();
-    m_videoOutput = output;
-    m_videoConnection = connection;
-
-    return true;
+    return [m_service->videoOutput()->videoDataOutput() connectionWithMediaType:AVMediaTypeVideo];
 }
 
 AVFCameraViewfinderSettingsControl::AVFCameraViewfinderSettingsControl(AVFCameraService *service)
