@@ -42,7 +42,7 @@
 #include <AudioUnit/AudioUnit.h>
 #include <AudioToolbox/AudioToolbox.h>
 #if defined(Q_OS_OSX)
-# include <CoreServices/CoreServices.h>
+# include <AudioUnit/AudioComponent.h>
 #endif
 
 #if defined(Q_OS_IOS)
@@ -222,6 +222,7 @@ CoreAudioOutput::CoreAudioOutput(const QByteArray &device)
     , m_startTime(0)
     , m_audioBuffer(0)
     , m_cachedVolume(1.0)
+    , m_pullMode(false)
     , m_errorCode(QAudio::NoError)
     , m_stateCode(QAudio::StoppedState)
 {
@@ -271,6 +272,7 @@ void CoreAudioOutput::start(QIODevice *device)
         m_stateCode = QAudio::ActiveState;
 
     // Start
+    m_pullMode = true;
     m_errorCode = QAudio::NoError;
     m_totalFrames = 0;
     m_startTime = CoreAudioUtils::currentTime();
@@ -296,6 +298,7 @@ QIODevice *CoreAudioOutput::start()
     m_stateCode = QAudio::IdleState;
 
     // Start
+    m_pullMode = false;
     m_errorCode = QAudio::NoError;
     m_totalFrames = 0;
     m_startTime = CoreAudioUtils::currentTime();
@@ -347,7 +350,7 @@ void CoreAudioOutput::resume()
     if (m_stateCode == QAudio::SuspendedState) {
         audioThreadStart();
 
-        m_stateCode = QAudio::ActiveState;
+        m_stateCode = m_pullMode ? QAudio::ActiveState : QAudio::IdleState;
         m_errorCode = QAudio::NoError;
         emit stateChanged(m_stateCode);
     }
@@ -540,30 +543,13 @@ bool CoreAudioOutput::open()
     if (m_isOpen)
         return true;
 
-#if defined(Q_OS_OSX)
-    ComponentDescription componentDescription;
-    componentDescription.componentType = kAudioUnitType_Output;
-    componentDescription.componentSubType = kAudioUnitSubType_HALOutput;
-    componentDescription.componentManufacturer = kAudioUnitManufacturer_Apple;
-    componentDescription.componentFlags = 0;
-    componentDescription.componentFlagsMask = 0;
-
-    // Open
-    Component component = FindNextComponent(NULL, &componentDescription);
-    if (component == 0) {
-        qWarning() << "QAudioOutput: Failed to find HAL Output component";
-        return false;
-    }
-
-    if (OpenAComponent(component, &m_audioUnit) != noErr) {
-        qWarning() << "QAudioOutput: Unable to Open Output Component";
-        return false;
-    }
-#else //iOS
-
     AudioComponentDescription componentDescription;
     componentDescription.componentType = kAudioUnitType_Output;
+#if defined(Q_OS_OSX)
+    componentDescription.componentSubType = kAudioUnitSubType_HALOutput;
+#else
     componentDescription.componentSubType = kAudioUnitSubType_RemoteIO;
+#endif
     componentDescription.componentManufacturer = kAudioUnitManufacturer_Apple;
     componentDescription.componentFlags = 0;
     componentDescription.componentFlagsMask = 0;
@@ -578,7 +564,6 @@ bool CoreAudioOutput::open()
         qWarning() << "QAudioOutput: Unable to Open Output Component";
         return false;
     }
-#endif
 
     // register callback
     AURenderCallbackStruct callback;
@@ -670,11 +655,7 @@ void CoreAudioOutput::close()
     if (m_audioUnit != 0) {
         AudioOutputUnitStop(m_audioUnit);
         AudioUnitUninitialize(m_audioUnit);
-#if defined(Q_OS_OSX)
-        CloseComponent(m_audioUnit);
-#else //iOS
         AudioComponentInstanceDispose(m_audioUnit);
-#endif
     }
 
     delete m_audioBuffer;
