@@ -42,13 +42,23 @@
 #endif // ANDROID
 
 #define BUFFER_COUNT 2
-#define DEFAULT_PERIOD_TIME_MS 50
-#define MINIMUM_PERIOD_TIME_MS 5
 #define EBASE 2.302585093
 #define LOG10(x) qLn(x)/qreal(EBASE)
 
 QT_BEGIN_NAMESPACE
 
+static inline void openSlDebugInfo()
+{
+    const QAudioFormat &format = QAudioDeviceInfo::defaultOutputDevice().preferredFormat();
+    qDebug() << "======= OpenSL ES Device info ======="
+             << "\nSupports low-latency playback: " << (QOpenSLESEngine::supportsLowLatency() ? "YES" : "NO")
+             << "\nPreferred sample rate: " << QOpenSLESEngine::getOutputValue(QOpenSLESEngine::SampleRate, -1)
+             << "\nFrames per buffer: " << QOpenSLESEngine::getOutputValue(QOpenSLESEngine::FramesPerBuffer, -1)
+             << "\nPreferred Format: "  << format
+             << "\nLow-latency buffer size: " << QOpenSLESEngine::getLowLatencyBufferSize(format)
+             << "\nDefault buffer size: " << QOpenSLESEngine::getDefaultBufferSize(format);
+}
+
 QMap<QString, qint32> QOpenSLESAudioOutput::m_categories;
 
 QOpenSLESAudioOutput::QOpenSLESAudioOutput(const QByteArray &device)
@@ -100,11 +110,18 @@ QAudio::State QOpenSLESAudioOutput::state() const
 void QOpenSLESAudioOutput::start(QIODevice *device)
 {
     Q_ASSERT(device);
+
+    if (m_state != QAudio::StoppedState)
+        stop();
+
     if (!preparePlayer())
         return;
 
     m_pullMode = true;
     m_audioSource = device;
+    m_nextBuffer = 0;
+    m_processedBytes = 0;
+    m_availableBuffers = BUFFER_COUNT;
     setState(QAudio::ActiveState);
     setError(QAudio::NoError);
 
@@ -122,6 +139,9 @@ void QOpenSLESAudioOutput::start(QIODevice *device)
         m_processedBytes += readSize;
     }
 
+    if (m_processedBytes < 1)
+        onEOSEvent();
+
     // Change the state to playing.
     // We need to do this after filling the buffers or processedBytes might get corrupted.
     startPlayer();
@@ -129,10 +149,15 @@ void QOpenSLESAudioOutput::start(QIODevice *device)
 
 QIODevice *QOpenSLESAudioOutput::start()
 {
+    if (m_state != QAudio::StoppedState)
+        stop();
+
     if (!preparePlayer())
         return Q_NULLPTR;
 
     m_pullMode = false;
+    m_processedBytes = 0;
+    m_availableBuffers = BUFFER_COUNT;
     m_audioSource = new SLIODevicePrivate(this);
     m_audioSource->open(QIODevice::WriteOnly | QIODevice::Unbuffered);
 
@@ -238,7 +263,7 @@ void QOpenSLESAudioOutput::resume()
         return;
     }
 
-    setState(QAudio::ActiveState);
+    setState(m_pullMode ? QAudio::ActiveState : QAudio::IdleState);
     setError(QAudio::NoError);
 }
 
@@ -358,7 +383,10 @@ void QOpenSLESAudioOutput::bufferAvailable(quint32 count, quint32 playIndex)
 
     if (!m_pullMode) { // We're in push mode.
         // Signal that there is a new open slot in the buffer and return
-        m_availableBuffers.fetchAndAddRelease(1);
+        const int val = m_availableBuffers.fetchAndAddRelease(1) + 1;
+        if (val == BUFFER_COUNT)
+            QMetaObject::invokeMethod(this, "onEOSEvent", Qt::QueuedConnection);
+
         return;
     }
 
@@ -366,8 +394,11 @@ void QOpenSLESAudioOutput::bufferAvailable(quint32 count, quint32 playIndex)
     const int index = m_nextBuffer * m_bufferSize;
     const qint64 readSize = m_audioSource->read(m_buffers + index, m_bufferSize);
 
-    if (1 > readSize)
+    if (readSize < 1) {
+        QMetaObject::invokeMethod(this, "onEOSEvent", Qt::QueuedConnection);
         return;
+    }
+
 
     if (SL_RESULT_SUCCESS != (*m_bufferQueueItf)->Enqueue(m_bufferQueueItf,
                                                           m_buffers + index,
@@ -531,13 +562,17 @@ bool QOpenSLESAudioOutput::preparePlayer()
 
     setVolume(m_volume);
 
+    const int lowLatencyBufferSize = QOpenSLESEngine::getLowLatencyBufferSize(m_format);
+    const int defaultBufferSize = QOpenSLESEngine::getDefaultBufferSize(m_format);
+
     // Buffer size
     if (m_bufferSize <= 0) {
-        m_bufferSize = m_format.bytesForDuration(DEFAULT_PERIOD_TIME_MS * 1000);
-    } else {
-        const int minimumBufSize = m_format.bytesForDuration(MINIMUM_PERIOD_TIME_MS * 1000);
-        if (m_bufferSize < minimumBufSize)
-            m_bufferSize = minimumBufSize;
+        m_bufferSize = defaultBufferSize;
+    } else if (QOpenSLESEngine::supportsLowLatency()) {
+        if (m_bufferSize < lowLatencyBufferSize)
+            m_bufferSize = lowLatencyBufferSize;
+    } else if (m_bufferSize < defaultBufferSize) {
+        m_bufferSize = defaultBufferSize;
     }
 
     m_periodSize = m_bufferSize;
@@ -588,6 +623,12 @@ void QOpenSLESAudioOutput::stopPlayer()
 {
     setState(QAudio::StoppedState);
 
+    if (m_audioSource && !m_pullMode) {
+        m_audioSource->close();
+        delete m_audioSource;
+        m_audioSource = Q_NULLPTR;
+    }
+
     // We need to change the state manually...
     if (m_playItf)
         (*m_playItf)->SetPlayState(m_playItf, SL_PLAYSTATE_STOPPED);
@@ -598,6 +639,9 @@ void QOpenSLESAudioOutput::stopPlayer()
 
 void QOpenSLESAudioOutput::startPlayer()
 {
+    if (QOpenSLESEngine::printDebugInfo())
+        openSlDebugInfo();
+
     if (SL_RESULT_SUCCESS != (*m_playItf)->SetPlayState(m_playItf, SL_PLAYSTATE_PLAYING)) {
         setError(QAudio::FatalError);
         destroyPlayer();
