@@ -38,8 +38,11 @@
 #define QWINRTCAMERACONTROL_H
 
 #include <QtMultimedia/QCameraControl>
+#include <QtCore/QLoggingCategory>
 #include <QtCore/qt_windows.h>
 
+#include <wrl.h>
+
 namespace ABI {
     namespace Windows {
         namespace Media {
@@ -57,10 +60,14 @@ namespace ABI {
 
 QT_BEGIN_NAMESPACE
 
+Q_DECLARE_LOGGING_CATEGORY(lcMMCamera)
+
 class QVideoRendererControl;
 class QVideoDeviceSelectorControl;
 class QCameraImageCaptureControl;
 class QImageEncoderControl;
+class QCameraFocusControl;
+class QCameraLocksControl;
 
 class QWinRTCameraControlPrivate;
 class QWinRTCameraControl : public QCameraControl
@@ -85,15 +92,29 @@ public:
     QVideoDeviceSelectorControl *videoDeviceSelector() const;
     QCameraImageCaptureControl *imageCaptureControl() const;
     QImageEncoderControl *imageEncoderControl() const;
+    QCameraFocusControl *cameraFocusControl() const;
+    QCameraLocksControl *cameraLocksControl() const;
 
-    ABI::Windows::Media::Capture::IMediaCapture *handle() const;
+    Microsoft::WRL::ComPtr<ABI::Windows::Media::Capture::IMediaCapture> handle() const;
+
+    bool setFocus(QCameraFocus::FocusModes mode);
+    bool setFocusPoint(const QPointF &point);
+    bool focus();
+    void clearFocusPoint();
+    void emitError(int errorCode, const QString &errorString);
+    bool lockFocus();
+    bool unlockFocus();
+    void frameMapped();
+    void frameUnmapped();
 
 private slots:
     void onBufferRequested();
+    void onApplicationStateChanged(Qt::ApplicationState state);
 
 private:
     HRESULT enumerateDevices();
     HRESULT initialize();
+    HRESULT initializeFocus();
     HRESULT onCaptureFailed(ABI::Windows::Media::Capture::IMediaCapture *,
                             ABI::Windows::Media::Capture::IMediaCaptureFailedEventArgs *);
     HRESULT onRecordLimitationExceeded(ABI::Windows::Media::Capture::IMediaCapture *);
